@@ -19,6 +19,7 @@ from __future__ import annotations
 from collections.abc import Callable
 
 from bleak.backends.device import BLEDevice
+from bleak.backends.service import BleakGATTServiceCollection
 from bleak_retry_connector import (
     BleakAbortedError,
     BleakConnectionError,
@@ -39,6 +40,7 @@ async def establish_connection(
     name: str,
     disconnected_callback: Callable[[AIOHomeKitBleakClient], None],
     max_attempts: int = MAX_CONNECT_ATTEMPTS,
+    cached_services: BleakGATTServiceCollection | None = None,
 ) -> AIOHomeKitBleakClient:
     """Establish a connection to the accessory."""
     try:
@@ -48,6 +50,7 @@ async def establish_connection(
             name,
             disconnected_callback,
             max_attempts=max_attempts,
+            cached_services=cached_services,
         )
     except (BleakAbortedError, BleakConnectionError) as ex:
         raise AccessoryDisconnectedError(ex) from ex
