@@ -16,6 +16,7 @@
 
 import asyncio
 import logging
+from typing import TYPE_CHECKING
 
 from aiohomekit.crypto.chacha20poly1305 import (
     ChaCha20Poly1305Decryptor,
@@ -23,6 +24,7 @@ from aiohomekit.crypto.chacha20poly1305 import (
 )
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
+    AccessoryNotFoundError,
     AuthenticationError,
     ConnectionError,
     HomeKitException,
@@ -38,6 +40,9 @@ from aiohomekit.utils import async_create_task
 
 logger = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    from aiohomekit.controller.ip.controller import IpController
+
 
 class InsecureHomeKitProtocol(asyncio.Protocol):
     def __init__(self, connection):
@@ -595,15 +600,29 @@ class SecureHomeKitConnection(HomeKitConnection):
         """_connect_once must only ever be called from _reconnect to ensure its done with a lock."""
         self.is_secure = False
 
-        """
-        try:
-            self.host, self.port = await async_find_device_ip_and_port(
-                self.pairing_data["AccessoryPairingID"],
-                async_zeroconf_instance=self.owner.controller._async_zeroconf_instance,
-            )
-        except AccessoryNotFoundError:
-            pass
-        """
+        if self.owner:
+            controller: IpController = self.owner.controller
+            try:
+                discovery = await controller.async_find(
+                    self.pairing_data["AccessoryPairingID"]
+                )
+                if self.host != discovery.description.address:
+                    logger.debug(
+                        "Host changed from %s to %s",
+                        self.host,
+                        discovery.description.address,
+                    )
+                    self.host = discovery.description.address
+
+                if self.port != discovery.description.port:
+                    logger.debug(
+                        "Port changed from %s to %s",
+                        self.port,
+                        discovery.description.port,
+                    )
+                    self.port = discovery.description.port
+            except AccessoryNotFoundError:
+                pass
 
         await super()._connect_once()
 
