@@ -103,7 +103,7 @@ class IpDiscovery(ZeroconfDiscovery):
 
         return finish_pairing
 
-    async def async_identify(self):
+    async def async_identify(self) -> None:
         await self._ensure_connected()
 
         response = await self.connection.post_json("/identify", {})
@@ -119,5 +119,3 @@ class IpDiscovery(ZeroconfDiscovery):
                 code=code.value,
             )
         )
-
-        return True
