@@ -17,12 +17,13 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from enum import Enum
-import json
 from typing import Any, Iterable, Iterator
 
+import aiohomekit.hkjson as hkjson
 from aiohomekit.protocol.statuscodes import to_status_code
 from aiohomekit.uuid import normalize_uuid
 
+from . import entity_map
 from .categories import Categories
 from .characteristics import (
     Characteristic,
@@ -40,6 +41,8 @@ __all__ = [
     "CharacteristicFormats",
     "FeatureFlags",
     "Accessory",
+    "Service",
+    "ServiceTypes",
     "Transport",
 ]
 
@@ -128,7 +131,7 @@ class Services:
 
 
 class Characteristics:
-    def __init__(self, services):
+    def __init__(self, services: Services) -> None:
         self._services = services
 
     def iid(self, iid: int) -> Characteristic | None:
@@ -297,11 +300,11 @@ class Accessories:
 
     @classmethod
     def from_file(cls, path) -> Accessories:
-        with open(path) as fp:
-            return cls.from_list(json.load(fp))
+        with open(path, encoding="utf-8") as fp:
+            return cls.from_list(hkjson.loads(fp.read()))
 
     @classmethod
-    def from_list(cls, accessories: list[dict[str, Any]]) -> Accessories:
+    def from_list(cls, accessories: entity_map.Accesories) -> Accessories:
         self = cls()
         for accessory in accessories:
             self.add_accessory(Accessory.create_from_dict(accessory))
@@ -310,15 +313,15 @@ class Accessories:
     def add_accessory(self, accessory: Accessory) -> None:
         self.accessories.append(accessory)
 
-    def serialize(self):
+    def serialize(self) -> entity_map.Accesories:
         accessories_list = []
         for a in self.accessories:
             accessories_list.append(a.to_accessory_and_service_list())
         return accessories_list
 
-    def to_accessory_and_service_list(self):
+    def to_accessory_and_service_list(self) -> dict[str, entity_map.Accesories]:
         d = {"accessories": self.serialize()}
-        return json.dumps(d)
+        return hkjson.dumps(d)
 
     def aid(self, aid) -> Accessory:
         return next(filter(lambda accessory: accessory.aid == aid, self.accessories))
